(**
The determinant space in which we solve the Schrodinger equation.
*)

type t

(** {1 Accessors} *)

val n_alfa : t -> int
(** Number of {% $\alpha$ %} electrons in the {% $\alpha$ %} MOs. *)

val n_beta : t -> int
(** Number of {% $\beta$ %} electrons in the {% $\beta$ %} MOs. *)

val mo_class : t -> MOClass.t
(** The MO classes used to generate the space. *)

val mo_basis : t -> MOBasis.t
(** The MO basis on which the determinants are expanded. *)

val determinants : t -> Determinant.t array
(** All the determinants belonging to the space. *)

val size : t -> int
(** Size of the determinant space *)


val fci_of_mo_basis : ?frozen_core:bool -> MOBasis.t -> t
(** Creates a space of all possible ways to put [n_alfa] electrons in the {% $\alpha$ %}
    [Active] MOs and [n_beta] electrons in the {% $\beta$ %} [Active] MOs.
    All other MOs are untouched.
*)

(** {2 Printing} *)

val pp_det_space : Format.formatter -> t -> unit
